<?php

/**
 * @file
 * Settings forms for Taxonomy Set Lineage.
 */

/**
 * Build the configuration form.
 *
 * @param array $form
 *   The configuration form.
 * @param array $form_state
 *   The configuration form state.
 */
function taxonomy_set_lineage_config_form(array $form, array &$form_state) {
  $settings = _taxonomy_set_lineage_get_settings();

  $form['intro'] = [
    '#markup' => t('Use the options below to set the scope for this module. You must select the vocabularies this module will be active for. You can then optionally limit it to specific entity types, bundles or fields.'),
  ];

  $form['taxonomy_set_lineage_affected_taxonomies'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Taxonomy Vocabularies'),
    '#options' => array(),
    '#description' => t('The lineage will be saved for the selected vocabularies. If none of the options below are selected then this will happen for all entity types, bundles or fields.'),
    '#default_value' => $settings['vocabularies'],
    '#required' => TRUE,
  );
  $form['taxonomy_set_lineage_affected_entities'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Entity Types'),
    '#options' => array(),
    '#description' => t('The lineage will be saved for the active vocabularies for the selected entity types.'),
    '#default_value' => $settings['entities'],
  );
  $form['taxonomy_set_lineage_affected_bundles'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Entity Bundles'),
    '#options' => array(),
    '#description' => t('The lineage will be saved for the active vocabularies for the selected entity bundles. This does not require the entity above to be selected.'),
    '#default_value' => $settings['bundles'],
  );
  $form['taxonomy_set_lineage_affected_fields'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Taxonomy Reference Fields'),
    '#options' => array(),
    '#description' => t('The linage will be saved for these specific fields only. This does not require you to select the corresponding entity or bundle.'),
    '#default_value' => $settings['fields'],
  );

  // Fill options with values.
  foreach (taxonomy_vocabulary_get_names() as $vocab) {
    $form['taxonomy_set_lineage_affected_taxonomies']['#options'][$vocab->machine_name] = check_plain($vocab->name);
  }

  $taxonomy_fields = field_read_fields(array('type' => 'taxonomy_term_reference'));
  $entity_reference_fields = field_read_fields(array('type' => 'entityreference'));

  foreach ($entity_reference_fields as $field_name => $info) {
    if ($info['settings']['target_type'] === 'taxonomy_term') {
      $taxonomy_fields[$field_name] = $info;
    }
  }

  foreach ($taxonomy_fields as $field_name => $info) {

    $base_info = field_info_field($field_name);

    if (!empty($base_info['bundles'])) {
      foreach ($base_info['bundles'] as $entity => $bundles) {
        foreach ($bundles as $bundle) {
          $field_instance = field_info_instance($entity, $field_name, $bundle);
          if (!empty($field_instance)) {
            $form['taxonomy_set_lineage_affected_fields']['#options'][$field_name] = check_plain($field_instance['label'] . ' (' . $field_name . ')');
            $entity_info = entity_get_info($entity);
            $form['taxonomy_set_lineage_affected_entities']['#options'][$entity] = check_plain($entity_info['label']);
            if (!empty($entity_info['bundles'][$bundle])) {
              $form['taxonomy_set_lineage_affected_bundles']['#options'][$bundle] = check_plain($entity_info['bundles'][$bundle]['label']);
            }
          }
        }
      }
    }
  }

  return system_settings_form($form);
}
